import { HexbinLayer, HexbinLayerConfig } from '../../leaflet-hexbin/src';
import { PropType, Ref, SetupContext } from 'vue';
import { LatLngExpression, LeafletEventHandlerFnMap } from 'leaflet';
type HoverHandlers = {
    fill?: boolean;
    scale?: number;
};
/**
 * Build fails when using ExtractPublicPropTypes so we resort to redefining prop types
 */
export type LHexbinLayerProps<Data> = {
    data: Data[];
    accessor?: (d: Data) => LatLngExpression;
    radius?: number;
    opacity?: number | [number, number];
    radiusRange?: [number, number];
    duration?: number;
    colorScaleExtent?: [number, number];
    radiusScaleExtent?: [number, number];
    opacityScaleExtent?: [number, number];
    colorRange?: string[];
    hover?: {
        fill?: boolean;
        scale?: number;
    };
};
export declare function hexbinLayerProps<Data>(): {
    readonly hover: {
        readonly type: PropType<HoverHandlers>;
    };
    /**
     * LatLng data points to be used for the hexbin layer.
     */
    readonly data: {
        readonly type: PropType<Array<Data>>;
    };
    /**
     * Accessor function for the data points if data is not LatLngExpression
     */
    readonly accessor: {
        readonly type: PropType<(d: Data) => LatLngExpression>;
    };
    /**
     * Hex grid cell radius in pixels.
     * This radius controls the radius of the hexagons used to bin the data
     * but not necessarily to draw each individual hexbin.
     */
    readonly radius: {
        readonly type: NumberConstructor;
        readonly validator: (value: number) => boolean;
    };
    /**
     * Opacity of the layer.
     */
    readonly opacity: {
        readonly type: PropType<number | [number, number]>;
        readonly validator: (value: number | [number, number]) => boolean;
        readonly default: 0.6;
    };
    /**
     * Duration of transition in milliseconds.
     */
    readonly duration: {
        readonly type: NumberConstructor;
        readonly validator: (value: number) => boolean;
    };
    /**
     * Color scale extent: [min, max].
     */
    readonly colorScaleExtent: {
        readonly type: PropType<[number, number]>;
        readonly default: readonly [1, undefined];
    };
    /**
     * Radius scale extent: [min, max].
     */
    readonly radiusScaleExtent: {
        readonly type: PropType<[number, number]>;
        readonly default: readonly [1, undefined];
    };
    /**
     * Opacity scale extent: [min, max].
     */
    readonly opacityScaleExtent: {
        readonly type: PropType<[number, number]>;
        readonly default: readonly [1, undefined];
    };
    /**
     * Sets the range of the color scale used to fill the hexbins.
     * Colors scale interpolates between all provided colors.
     */
    readonly colorRange: {
        readonly type: {
            (arrayLength: number): string[];
            (...items: string[]): string[];
            new (arrayLength: number): string[];
            new (...items: string[]): string[];
            isArray(arg: any): arg is any[];
            readonly prototype: any[];
            from<T>(arrayLike: ArrayLike<T>): T[];
            from<T, U>(arrayLike: ArrayLike<T>, mapfn: (v: T, k: number) => U, thisArg?: any): U[];
            from<T>(iterable: Iterable<T> | ArrayLike<T>): T[];
            from<T, U>(iterable: Iterable<T> | ArrayLike<T>, mapfn: (v: T, k: number) => U, thisArg?: any): U[];
            of<T>(...items: T[]): T[];
            readonly [Symbol.species]: ArrayConstructor;
        };
        readonly default: readonly ["#f7fbff", "#08306b"];
    };
    /**
     * Sets the range of the radius scale used to size the hexbins.
     */
    readonly radiusRange: {
        readonly type: PropType<[number, number] | null>;
    };
    readonly pane: {
        readonly type: StringConstructor;
    };
    readonly attribution: {
        readonly type: StringConstructor;
    };
    readonly name: {
        readonly type: StringConstructor;
        readonly custom: true;
    };
    readonly layerType: {
        readonly type: import('vue').PropType<import('@vue-leaflet/vue-leaflet/dist/src/types/enums/LayerType').LayerType>;
        readonly custom: true;
    };
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly custom: true;
        readonly default: true;
    };
    readonly options: {
        readonly type: ObjectConstructor;
        readonly default: () => {};
        readonly custom: true;
    };
};
export declare const hexbinLayerEvents: {
    ready(_: HexbinLayer): boolean;
    click(): boolean;
    dblclick(): boolean;
};
export type LHexbinLayerEvents = typeof hexbinLayerEvents & LeafletEventHandlerFnMap;
export declare const setupHexbinLayer: <Data = LatLngExpression, Events = unknown>(props: LHexbinLayerProps<Data>, leafletRef: Ref<HexbinLayer<Data> | undefined>, context: SetupContext<Events>) => {
    options: HexbinLayerConfig;
    methods: {
        bindTooltip(leafletObject: any): void;
        unbindTooltip(): void;
        setRadius(radius: number): void;
        setDuration(duration: number): void;
        setOpacity(opacity: number | [number, number]): void;
        setColorScaleExtent(extent: [number, number]): void;
        setRadiusScaleExtent(extent: [number, number]): void;
        setColorRange(range: string[]): void;
        setRadiusRange(range: [number, number] | null): void;
        setData(data: Data[]): void;
        setAccessor(accessor: (d: Data) => LatLngExpression): void;
        addLayer(layer: any): void;
        removeLayer(layer: any): void;
        setAttribution(val: any): void;
        setName(): void;
        setLayerType(): void;
        setVisible(isVisible: any): void;
        bindPopup(leafletObject: any): void;
        unbindPopup(): void;
        updateVisibleProp(value: any): void;
    };
};
export {};
